function Z = gauss2(width, sigma, theta, amplitude, extent, padding)
% GAUSS2. Make a 2D Gaussian filter
%
% INPUT
% -------------
% width 
%     - empty | scalar | vector ( [] | w | [wy, wx] )
%     - filter width in y & x directions, in pixels
% sigma 
%     - empty | scalar | vector ( [] | s | [sy, sx] )
%     - default: [1 1]
%     - standard deviations along principal axes
%     - input optional
% theta 
%     - empty | scalar
%     - default: 0
%     - orientation in degrees of principal axis in degrees 
%         in respect to the x-axis, positive values  
%         increasing counterclockwise
%     - input optional
% amplitude 
%     - empty | scalar
%     - default: 1
%     - maximum value
%     - input optional
% extent 
%     - empty | scalar | vector ( [] | e | [emin, emax] )
%     - default: [-5 5] (smoothing nicely towards zero)
%     - extent from mean in standard deviation units
%         where the filter becomes zero
%     - input optional
% padding 
%     - empty | scalar | vector ( [] | p | [py, px] )
%     - default: [] (no padding)
%     - padding the filter with zero values; the filter will
%         be centered on a matrix of size py-by-px
%     - input optional
%     
% OUTPUT
% -------------
% Z - the filter matrix
%
% Note: If sy = sx the filter is isotropic.
% 
% EXAMPLE
% -------------
% Z = gauss2(47, [2 1], -30, 100, [-5 5]);
% figure, imagesc(Z), axis off image, colormap('spring')
% 
% CREDITS
% -------------
% Initial source: Gaussian function, Wikipedia 2010.09.09
% http://en.wikipedia.org/wiki/Gaussian_function
% Modified by: Vlad Atanasiu 2010.09.09

% set defaults
if nargin < 2 || isempty(sigma)
    sigma = [1 1];
end
if nargin < 3 || isempty(theta)
    theta = 0;
end
if nargin < 4 || isempty(amplitude)
    amplitude = 1;
end
if nargin < 5 || isempty(extent)
    extent = [-5 5];
end
if nargin < 6 || isempty(padding)
    padding = [];
end
theta = theta - 90;
theta = theta*pi/180;

% single input values make isotropic kernel
if length(width) == 1
    width = [width width];
end
if length(sigma) == 1
    sigma = [sigma sigma];
end
if length(extent) == 1
    extent = [extent extent];
end
if length(padding) == 1
    padding = [padding padding];
end

% prepare matrix
y0 = 0; x0 = 0; 
y = extent(1) : extent(2)*2 / (width(1)-1) : extent(2);
x = extent(1) : extent(2)*2 / (width(2)-1) : extent(2);
[X, Y] = meshgrid(x, y);

% compute coefficients
a = cos(theta)^2/2 / sigma(2)^2 + sin(theta)^2/2 / sigma(1)^2;
b = -sin(2*theta)/4 / sigma(2)^2 + sin(2*theta)/4 / sigma(1)^2;
c = sin(theta)^2/2 / sigma(2)^2 + cos(theta)^2/2 / sigma(1)^2;

% compute amplitude of matrix values
Z = amplitude*exp( - (a*(X-x0).^2 + 2*b*(X-x0).*(Y-y0) + c*(Y-y0).^2));

% padd kernel with zero values
if isempty(padding) ~= 1
    P = zeros(padding(1), padding(2));
    py = floor((padding(1) - width(1)) / 2);
    px = floor((padding(2) - width(2)) / 2);
    P(py+1:py+width(1), px+1:px+width(2)) = Z;
    Z = P;
end
