function or_viz(url,id,fig_save,fig_show,inflections)
% OR_VIZ Vizualize raw and pdf data of shapes local orientation
% 
% Note: This is a subfunction of or_wrap.
% 
% -------------
% INPUT
% -------------
% url - url of text file with image files names
%     format: filename \n filename ...
% id - row of file name to be visualized;
%     scalar or vector or string {'all'}
%     - if no id is provided all images will be processed
% fig_save - string: 'save' | {'nosave'}
%     - saves the colorcoded orientation image
% fig_show - string: 'show' | {'noshow'}
%     - shows the images
%     - Note: When batching it is desirable to set 'noshow'
%     to avoid creating too many figures on the screen
% inflections - string: 'inflections' | {'noinflections'}
%     - shows inflections points (second derivative zero-corssings)
% 
% -------------
% OUTPUT
% -------------
% Figure representing the raw and pdf data.
%     - save location: 'pix' directory relative to urlr file location
% Figure showing the profiles of the entire sample
%     - save location: 'dat' directory relative to urlr file location
% 
% -------------
% EXAMPLE
% -------------
% or_viz('batch.txt',[1:2],'save','show','noinflections')
% or_viz('batch.txt',[1:2],'save','show','inflections')
% or_viz('batch.txt',[1:1538],'save','noshow','noinflections')
% or_viz('batch.txt',[1:1538],'save','noshow','inflections')
% or_wrap('batch.txt','nopreprocess','save') % pre-function
%
% -------------
% CREDITS
% -------------
% Vlad Atanasiu
% atanasiu@alum.mit.edu | http://www.waqwaq.info/atanasiu/
% 2011.01.20 - [new] add save option
% 2010.12.27 - creation

% parse input
if nargin < 2
    id = 'all'; % all data will be processed
end
if nargin < 3
    fig_save = 'nosave';
end
if nargin < 4
    fig_show = 'noshow';
end
if nargin < 5
    inflections = 'noinflections';
end

% initializations
tstart = datestr(now);
pixdir = 'pix/';
datdir = 'dat/';

% construct data files urls
[fpath, fname] = fileparts(url);
% file is in the current workdirectory
if ~isempty(fpath)
    fpath = [fpath '/'];
end
% prepare results directory
if strcmp(fig_save,'save')
    if exist([fpath pixdir],'dir') == 0
        mkdir([fpath pixdir])
    end
end

% read data files
F = importdata(url);
T = importdata([fpath datdir fname '_or_frq.txt']);
P = importdata([fpath datdir fname '_or_pdf.txt']);
T = T.data;
P = P.data;

% scale data
% (should be the same for all figures to allow comparison)
if ischar(id) == 1
    id = 1:size(F,1);
end
z = length(id);
T_ext = [NaN NaN];
for k = 1:z
    % scale data to supperpose frequencies & pdf
    d = mean(T(id(k),:))-mean(P(id(k),:));
    T(id(k),:) = T(id(k),:)-d;
    T(id(k),:) = ( std(P(id(k),:))/std(T(id(k),:)) ) * ...
        T(id(k),:) + mean(P(id(k),:));
    T(id(k),:) = fliplr(T(id(k),:));
    P(id(k),:) = fliplr(P(id(k),:));
    
    % get extreme values for scaling the figures' axis
    T_ext = [ min([T(id(k),:) T_ext(1)]) ...
        max([T(id(k),:) T_ext(2)]) ];
end

% show selection as colorcoded image
fn = '';
if size(F,1) ~= 1
    % if only one image put its name on the figure
    f = char(F(id(k),:));
    [~, fn, fe] = fileparts(f);
    fn = [fn fe ' / '];
end
h_mtx = figure('name',[fn 'Pdf matrix']);
imagesc(P(id,:))
del = '||||||||||||||||||||';
num = ['|||||||||' '-80' del '-60' del '-40' del '-20' del ...
    '0' del '20' del '40' del '60' del '80' '||||||||||'];
set(gca,'TickDir','out','XTick',1:1:180,...
    'XTickLabel',num);
axis square
colormap(cool)
colorbar
title('Colorcoded pdf-s of selected images')
xlabel('Angle (degrees)')
ylabel('Script sample id')

% save figure
if strcmp(fig_save,'save')
    imwrite(P(id,:)*255,cool,[fpath datdir fname '_or_mtx.tif'],'tiff')
    saveas(h_mtx,[fpath datdir fname '_or_mtx.png'],'png')
    saveas(h_mtx,[fpath datdir fname '_or_mtx.fig'],'fig')
end
    
% close figure
if strcmp(fig_show,'noshow')
    delete(h_mtx)
end

% display waitbar
h_wb = waitbar(0,'');
set(h_wb,'Name','The machine is working... Drink coffee.')
waitbar(1/z, h_wb, ['1 / ' num2str(z) ' images'])
drawnow

% display individual items
for k = 1:z
    % select desired entry
    f = char(F(id(k),:));
    [~, fname, fext] = fileparts(f);

    % display image
    I = imread(f);
    if strcmp(fig_show,'show')
        h_img = figure('name',[fname fext ' image']);
        imagesc(I), colormap(bone), axis off image
        title(['Input image ' [fname, fext]])
    end
    
    % display measurements
    T_ext_range = T_ext(2) - T_ext(1);
    h_pdf = figure('name',[fname fext ' pdf']);
    axis([-90 91 ...
        T_ext(1)-T_ext_range*.05 ...
        T_ext(2)+T_ext_range*.05])
    line([-90 -90],get(gca,'ylim')+[eps 0],'Color','w')
    title(['Frequency count & pdf orientations of ' ...
        [regexprep(fname,'\_','\\_') fext]])
    hold on
    line([0 0],...
        [T_ext(1)-T_ext_range*.07 ...
        T_ext(2)+T_ext_range*.05],...
        'Color',[.5 .5 .5],'linestyle',':')
    plot(-89:90,T(id(k),:),'Color',[.5 .5 .5])
    plot(-89:90,P(id(k),:),'r','linewidth',2)
    
    % display inflection points
    if strcmp(inflections,'inflections')
        p = P(id(k),:);
        p = [p(end) p];
        d2 = abs(sign(diff(sign(diff(diff(p))))));
        d2 = d2.*(1:length(d2));
        d2(d2 == 0) = [];
        plot(d2-89-1,p(d2+1),...
            'LineStyle','none','linewidth',2,...
            'Marker','.','color','b')
    end
    
    % save pdf
    if strcmp(fig_save,'save')
        saveas(h_pdf,[fpath pixdir '/' fname '_or_pdf.png'],'png')
        saveas(h_pdf,[fpath pixdir '/' fname '_or_pdf.fig'],'fig')
    end
    
    % close figure
    if strcmp(fig_show,'noshow')
        delete(h_pdf)
    end
    
    % increment waitbar
    waitbar(k/z, h_wb, [num2str(k) ' / ' num2str(z) ' images'])
end

% close waitbar
close(h_wb)

% tell user where to find the results
if strcmp(fpath,'/')
    fpath = pwd;
end 
tend = datestr(now,'HH:MM:SS');
s = [...
    {[tstart ' > ' tend]} ...
    {''} ...
    ];
if strcmp(fig_save,'save')
    s = [s ...
        {['Results saved to:']} ...
        {''} ...
        {['Data in ' fpath datdir]} ...
        {['1. Orientations matrix image: ' fname '_or_mtx.png']} ...
        {['2. Orientations matrix Matlab figure: ' fname '_or_mtx.fig']} ...
        {''} ...
        {['Pictures in ' fpath pixdir]} ...
        {'3. Orientations distributions images: *_or_pdf.png'} ...
        {'4. Orientations distributions Matlab figures: *_or_pdf.png'} ...
        {''} ...
        ];
end
warndlg(s,'or_viz');

