function or_wrap(urlr,preproc,fig_save,fig_show,stroke_type,sort2key)
% OR_WRAP Wrapper for the measurements local shapes orientation
% 
% -------------
% INPUT
% -------------
% urlr - A string providing the url of a text file containing the 
%     list of images to process; if argument is empty you will 
%     be prompted to interactively select the files list (batch mode)
%     or a single image file to process (single mode).
% preproc   - string: {'preprocess'} | 'nopreprocess'
%           - option to preprocess the input images:
%           binarization, negative, [resizing]
% fig_save - string: {'save'} | 'nosave'
%     - saves the colorcoded orientation image & population pdf
% fig_show - string: 'show' | {'noshow'}
%     - shows the population distributions
%     - save location: same directory as urlr
% stroke_type - string: {'outlines'} | 'inline'
%     - extract the outlines or inline of the shape
% sort2key - url of a text file containing data to be used as
%     second sorting key for the figures, such as writer id
%     - format: filename [same as in urlr] tab scalar tab ... \n
% 
% -------------
% OUTPUT
% -------------
% 1. Pictures ('pix' directory relative to urlr file location)
%     *_or_cmap.png - colorcoded images of the orientations
%     *_or_pdf.png - orientation profiles
%     *_or_pdf.fig - orientation profiles (Matlab figures)
% 2. Measurements ('dat' directory relative to urlr file location)
%     {url}_frq.txt - frequency count orientation data
%     {url}_pdf.txt - probability density function data
%     {url}_stat.txt - statistics of individual samples
%     {url}_pop.txt - population statistics
% 
% -------------
% EXAMPLE
% -------------
% or_wrap('batch\batch.txt','preprocess','save','show')
% or_wrap('batch\batch.txt','preprocess','save','show','inline')
% or_wrap('batch\batch.txt','nopreprocess','save','show')
% or_wrap('iam\iam_off.txt','nopreprocess','save','show','outlines','iam\iam_off_wri.txt')
%
% -------------
% ATTENTION
% -------------
% File names of images starting with a non-numerical character will have
% some characters preped.
%
% -------------
% SEE ALSO
% -------------
% To visualize the frequency count and pdf of a specific
% image from the data files generate with this function use
% or_viz(url,i), where 'i' is the i-th image in the url file.
% 
% More about the contour orientation capabilities and applications on 
% http://glyph.telecom-paristech.fr/rex/.
%
% -------------
% CREDITS
% -------------
% Vlad Atanasiu
% atanasiu@alum.mit.edu | http://alum.mit.edu/www/atanasiu/
% 2011.08.05 - [new] activate outlines/inline option
% 2011.02.28 - [mod] includes or_viz to create distribution plots
% 2011.01.20 - [new] add save option for colorcoded orientation map
% 2010.12.27 - creation


% parse input
if nargin < 1
    [fname, fpath] = uigetfile(...
       {'*.txt', 'Images list (*.txt)'}, 'REX - Load images');
    urlr = [fpath, fname];
end
if nargin < 2
    preproc = 'preprocess';
end
if nargin < 3
    fig_save = 'save';
end
if nargin < 4
    fig_show = 'noshow';
end
if nargin < 5
    stroke_type = 'outlines';
end
if nargin < 6
    sort2key = [];
end

% initializations
tstart = datestr(now);
pixdir = 'pix/';
datdir = 'dat/';

% prepare to make file names to be created by this function
[fpath, fname] = fileparts(urlr);
% file is in the current workdirectory
if ~isempty(fpath)
    fpath = [fpath '/'];
else
    fpath = [pwd '/'];
end

% delete any previous orientation files with the same name
s = {'frq' 'pdf' 'stat' 'pop'};
for k = 1:length(s)
    url_w = cell2mat([fpath datdir fname '_' s(k) '.txt']);
    if exist(url_w,'file') ~= 0
        delete(url_w)
    end
end

% display waitbar
wbn = 3;
% wbn = 5;
wbi = 0;
h_wb_wrap = waitbar(0,'');
h_wb_wrap_pos = get(h_wb_wrap,'OuterPosition');
set(h_wb_wrap,'Name','The machine is working... Drink coffee.',...
    'OuterPosition',h_wb_wrap_pos+[0 h_wb_wrap_pos(4)*3 0 0])
waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
drawnow


% frequency count orientation data
wbi = wbi+1;
waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
or_frq(urlr,preproc,fig_save,stroke_type)

% pdf of raw orientations
wbi = wbi+1;
waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
or_pdf(urlr)

% % statistics of individual samples pdf-s
% wbi = wbi+1;
% waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
% or_stat(urlr)

% plots showing the orientations' frequency count and pdf-s
wbi = wbi+1;
waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
or_viz(urlr,'all',fig_save,'noshow')

% % statistics of population pdf-s & representation of results
% wbi = wbi+1;
% waitbar(wbi/wbn, h_wb_wrap,[num2str(wbi) ' / ' num2str(wbn) ' steps'])
% or_pop(urlr,fig_save,fig_show,sort2key)


% close waitbar
close(h_wb_wrap)

% tell user where to find the results
tend = datestr(now,'HH:MM:SS');
s = [...
    {[tstart ' > ' tend]} ...
    {['Results saved to:']} ...
    {''} ...
    {['Data in ' fpath datdir]} ...
    {['1. Frequency count of orientations: ' fname '_frq.txt']} ...
    {['2. Probability density functions: ' fname '_pdf.txt']} ...
    {''} ...
    ];
%     {['3. Statistics of individual samples: ' fname '_stat.txt']} ...
%     {['4. Population statistics: ' fname '_pop.txt']} ...
%     {''} ...
%     ];
if strcmp(fig_save,'save')
    s = [s ...
        {['Pictures in ' fpath pixdir]} ...
        {'3. Colorcoded orientations: *_cmap.png'} ...
        {''} ...
        ];
end
warndlg(s,'or_wrap');


